# Copyright (c) 2008 Drew Smathers.
# See LICENSE for details
from pyglet import window
from pyglet import font
from pyglet.window import key

from itertools import cycle as itercycle

w = window.Window(640, 400)
f = font.load('Arial', 10)

class Runtime:
    pass

class LoopingRuntime(Runtime):
    def run(self):
        while not w.has_exit:
            w.dispatch_events()
            w.clear()
            w.flip()
        w.close()

class ActorRuntime(Runtime):
    """The runtime sets up a cooiterator for display
    """
    def run(self):
        raise NotImplementedError, 'sorry ... not done'


### Put in other file

# There has to be a nicer way of getting the key pressed

from cStringIO import StringIO

class TextPanel(object):

    def __init__(self, fontface='Arial', fontsize=12, color=(1,1,1), leading=10):
        self.font = font.load(fontface, fontsize)
        self.offset = self.font.ascent + self.font.descent + leading
        self.current = [5,5]

class LineEditor(object):
    """Basic line editor
    """

    def __init__(self, typed_limit=100, history_limit=500):
        self.typed = []
        self.history = []
        self.typed_limit = typed_limit
        self.history_limit = history_limit
        self._lhs_buffer = StringIO()
        self._rhs_buffer = StringIO()
        self.history_iter = self._bidir_iter()
        self._cached = ''

    def enter(self):
        t, h = self.typed, self.history
        lb, rb = self._lhs_buffer, self._rhs_buffer
        line = lb.getvalue() + rb.getvalue()
        t.append(line)
        h.append(line)
        if len(t) > self.typed_limit: t.pop(0)
        if len(h) > self.history_limit: h.pop(0)
        self.history_iter = self._bidir_iter()
        for b in lb, rb:
            b.reset()
            b.truncate()
            b.flush()
        self._cache = ''

    def _bidir_iter(self):
        """Bidirecation iterator over typed history
        """
        h = list(self.history), []
        h.reverse()
        direction = 1
        pos = 0
        while True:
            for elm in itercycle(list(h)):
                ndir = yield elm
                if direction != ndir:
                    h.reverse()
                    direction = ndir
                    h = h[:len(h) - pos] + h[len(h) - pos:]
                    break
                pos += 1
                pos %= len(h)

    def __str__(self):
        return self._cached

    def _rhs_str(self):
        ls = list(self._rhs_buffer.getvalue())
        ls.reverse()
        return ''.join(ls)

    def _recache(self):
        self._cached = self._lhs_buffer.getvalue() + self._rhs_str()

    def on_key_press(self, symbol, modifiers):
        if symbol == key.ENTER:
            self.enter()
            self._cached = ''
            print 'buffer :', str(self)
        elif symbol == key.ESCAPE:
            w.has_exit = True
        
    def on_text(self, text):
        if text == '\r': # XXX
            return
        self._lhs_buffer.write(text)
        self._recache()
        print 'buffer :', str(self)

    def _shift(self, b1, b2):
        b1.seek(b1.tell() - 1)
        c = b1.read(1)
        b1.seek(b1.tell() - 1)
        b1.truncate()
        b2.write(c)

    def _delete(self, b):
        b.seek(b.tell() - 1) # might not work right with unicode
        b.truncate()
        self._recache()

    def on_text_motion(self, motion):
        if motion == key.MOTION_BACKSPACE:
            self._delete(self._lhs_buffer)
        if motion == key.MOTION_DELETE:
            self._delete(self._rhs_buffer)
        elif motion == key.MOTION_LEFT:
            self._shift(self._lhs_buffer, self._rhs_buffer)
        elif motion == key.MOTION_RIGHT:
            self._shift(self._rhs_buffer, self._lhs_buffer)
        print 'buffer :', str(self)

lineEditor = LineEditor()
w.on_key_press = lineEditor.on_key_press
w.on_text = lineEditor.on_text
w.on_text_motion = lineEditor.on_text_motion

if __name__ == '__main__':
    import sys
    if sys.argv[1:]:
        klass = sys.argv[1]
        runtime = globals()[klass]()
    else:
        runtime = LoopingRuntime()
    runtime.run()



