from miru.test import unittest, require_window
from miru import camera
from miru import imiru
from miru import components
from miru import mesh

require_window()

class LightGroupTest(unittest.TestCase):

    def testWithinLightsLimit(self):
        lightGroup = camera.LightGroup()

        # add up to N supported lights
        for i in range(camera.LIGHT_CT_MAX):
            lightGroup.append(camera.DirectionalLight())
        # Ok ..

    def testExceedingLightsLimit(self):

        def overload():
            lightGroup = camera.LightGroup()
            for i in range(camera.LIGHT_CT_MAX + 1):
                lightGroup.append(camera.DirectionalLight())

        self.failUnlessRaises(AssertionError, overload)

class TestBlittableView(unittest.TestCase):

    def testZOrdering(self):
        bv = components.getUtility(imiru.IBlittableRenderStage)
        o1 = mesh.Object(pos=(0,0,1))
        o2 = mesh.Object(pos=(0,0,-1))
        o3 = mesh.Object(pos=(0,0,0))
        bv.addobj(o1)
        bv.addobj(o2)
        bv.addobj(o3)
        self.assertEquals([o2,o3,o1],bv.objects)
        o1.pos -= (0,0,3)
        self.assertEquals([o1,o2,o3],bv.objects)
        o2.pos += (0,0,5)
        self.assertEquals([o1,o3,o2],bv.objects)



