from miru.test import unittest
from miru import widgets as w
from miru import mesh
from miru import imiru

class TestWidget:

    def __init__(self, width, height):
        self.width = width
        self.height = height

class WidgetsTest(unittest.TestCase):

    def testColorBar(self):

        cb = w.ColorBar(top=(1,1,1,1), bottom=(0,0,0,1))
        self.assertEquals((1,1,1,1), cb.top_left)
        self.assertEquals((1,1,1,1), cb.top_right)
        self.assertEquals((0,0,0,1), cb.bottom_left)
        self.assertEquals((0,0,0,1), cb.bottom_right)

        cb = w.ColorBar(top_left=(1,1,0.1,1), top_right=(0,0.5,1,0.5), bottom=(0,0,0,1))
        self.assertEquals((1,1,0.1,1), cb.top_left)
        self.assertEquals((0,0.5,1,0.5), cb.top_right)
        self.assertEquals((0,0,0,1), cb.bottom_left)
        self.assertEquals((0,0,0,1), cb.bottom_right)
        
        cb = w.ColorBar(bottom_left=(1,1,0.1,1), bottom_right=(0,0.5,1,0.5), top=(0,1,0,1))
        self.assertEquals((0,1,0,1), cb.top_left)
        self.assertEquals((0,1,0,1), cb.top_right)
        self.assertEquals((1,1,0.1,1), cb.bottom_left)
        self.assertEquals((0,0.5,1,0.5), cb.bottom_right)
        
        cb = w.ColorBar(bottom_left=(1,1,0.1,1), bottom_right=(0,0.5,1,0.5),
                top_left=(0,1,0,1), top_right=(1,0,0,0.5))
        self.assertEquals((0,1,0,1), cb.top_left)
        self.assertEquals((1,0,0,0.5), cb.top_right)
        self.assertEquals((1,1,0.1,1), cb.bottom_left)
        self.assertEquals((0,0.5,1,0.5), cb.bottom_right)
        
        cb = w.ColorBar(left=(1,1,1,1), right=(0,0,0,1))
        self.assertEquals((1,1,1,1), cb.top_left)
        self.assertEquals((0,0,0,1), cb.top_right)
        self.assertEquals((1,1,1,1), cb.bottom_left)
        self.assertEquals((0,0,0,1), cb.bottom_right)
        
        cb = w.ColorBar(left=(1,1,1,1), top_right=(1,0,1,0.5), bottom_right=(0,0,0,1))
        self.assertEquals((1,1,1,1), cb.top_left)
        self.assertEquals((1,0,1,0.5), cb.top_right)
        self.assertEquals((1,1,1,1), cb.bottom_left)
        self.assertEquals((0,0,0,1), cb.bottom_right)


    def testLRBox(self):
        box = w.LRBox(objects=[TestWidget(5,5), TestWidget(5,10),
            TestWidget(10,7)]).pack(None)
        self.assertEquals(40, box.width)
        self.assertEquals(20, box.height)

    def testTBBox(self):
        box = w.TBBox(objects=[TestWidget(5,5), TestWidget(5,10),
            TestWidget(10,10)]).pack(None)
        self.assertEquals(20, box.width)
        self.assertEquals(45, box.height)

    def testBoxPadding(self):
        box = w.LRBox(objects=[TestWidget(5,5), TestWidget(5,10),
            TestWidget(10,7)], padding=2).pack(None)
        self.assertEquals(28, box.width)
        self.assertEquals(14, box.height)

        box = w.TBBox(objects=[TestWidget(5,5), TestWidget(5,10),
            TestWidget(10,10)], padding=3).pack(None)
        self.assertEquals(16, box.width)
        self.assertEquals(37, box.height)

    def testBoxNested(self):
        top = w.TBBox(objects=[TestWidget(10,5), TestWidget(10,10)])
        bottom = w.LRBox(objects=[TestWidget(5,5), TestWidget(7,7)])
        box = w.TBBox(objects=[top, bottom]).pack(None)
        self.assertEquals(47, box.height)
        self.assertEquals(27, box.width)
        
        left = w.TBBox(objects=[TestWidget(10,5), TestWidget(10,10)])
        right = w.LRBox(objects=[TestWidget(5,5), TestWidget(7,7)])
        box = w.LRBox(objects=[left, right]).pack(None)
        self.assertEquals(30, box.height)
        self.assertEquals(47, box.width)


    def testPane(self):
        top = w.TBBox(objects=[TestWidget(10,5), TestWidget(10,10)])
        bottom = w.LRBox(objects=[TestWidget(5,5), TestWidget(7,7)])
        box = w.TBBox(objects=[top, bottom])
        pane = w.Pane(box=box).pack()
        self.assertEquals(47, pane.height)
        self.assertEquals(27, pane.width)

    def testBoxHomogeneity(self):
        def makebox():
            box = w.LRBox(objects=[w.TBBox(objects=[TestWidget(1,1)]), TestWidget(2,2)])
        self.assertRaises(AssertionError, makebox)

    def testWidgetAdaption(self):
        o = mesh.newObject(w.Pane, box=None)
        wrapped = o.drawable
        adapted = imiru.IWidget(o)
        self.assertEquals(adapted, wrapped)

    def testWidgetAdaptionFailure(self):
        def tryAdapting():
            o = mesh.newObject(mesh.Rectangle)
            adapted = imiru.IWidget(o)
        self.assertRaises(TypeError, tryAdapting)

