# Copyright (c) 2008 Drew Smathers.
# See LICENSE for details
from pyglet.gl import *
from pyglet import window
from pyglet.window import key

import miru.mesh
from miru.editor import toggle_console
from miru.environment import env
from miru.utils import safepop

import random

from itertools import cycle

RANDOM_BENCHMARK = True
_bmark_choices = [ 'alien', 'woodball' ]

class BaseWindow(window.Window):

    def __init__(self, *p, **kw):
        self.clear_color = safepop(kw, 'clear_color', (0,0,0,1))
        self.effects = safepop(kw, 'effects', [])
        super(BaseWindow, self).__init__(*p, **kw)

    def on_resize(self, w, h):
        glViewport(0, 0, w, h)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        h = h or 1
        gluPerspective(50., w / float(h), 0.1, 1000.)
        glMatrixMode(GL_MODELVIEW)
        fNoLight = (4 * GLfloat)()
        fNoLight[:] = [0,0,0,0]
        glLightModelfv(GL_LIGHT_MODEL_AMBIENT, fNoLight);
        glClearColor(*self.clear_color)
        for effect in self.effects:
            effect.enable()
    

class TestWindow(BaseWindow):

    def __init__(self, *p, **kw):
        super(TestWindow, self).__init__(*p, **kw)

    def _load_object(self):
        if RANDOM_BENCHMARK:
            fname = 'docs/demo/%s.obj' % random.choice(_bmark_choices)
            o = miru.mesh.loadObj(fname)
            print 'loaded [random]', o
        else:
            o = miru.mesh.loadObj('docs/demo/alien.obj')
            print 'loaded', o
        env.addobj(o)
        return o

    bkey = _load_object

    def _on_key_press(self, pressed, modifiers):
        self._key_pressed = pressed
        self._modifiers = modifiers
        if pressed == key.T and (modifiers & key.MOD_CTRL):
            toggle_console()
        elif pressed == key.ESCAPE:
            self.has_exit = True
        elif pressed == key.B:
            self.bkey()
        #print pressed, modifiers


