# djfroofy's actions

from miru.environment import env
from miru import common

from roboto import actor
from roboto import scene

clock = scene.scheduler

import random
import euclid



class Accellerator(actor.BaseAction):
    acceleration = euclid.Vector3(20.,0.,-10.0)
    angle_acceleration = euclid.Vector3(0.,0.,0.0)
    pos_breaking = False
    angle_breaking = False

    actor = None

    def update(self, dt):

        actr = self.actor
        o = self.actor.actor_obj

        if self.acceleration is not None:
            a = self.acceleration
            v = (a[0] * dt, a[1] * dt, a[2] * dt)
            actr.velocity += v
            o.pos += (actr.velocity * dt)
        if self.angle_acceleration is not None:
            a = self.angle_acceleration
            v = (a[0] * dt, a[1] * dt, a[2] * dt)
            actr.angle_velocity += v
            o.angle += (actr.angle_velocity * dt)
        if self.tryGoal(dt):
            clock.unschedule(self.update)
            self.stop()
            if not self.deferred.called:
                self.deferred.callback(self)

    def tryGoal(self, dt):
        return False


class JumpAction(actor.BaseAction):
    

    def __init__(self, actor, vy):
        self.actor = actor
        self.vy = vy

    def start(self):
        d = super(JumpAction, self).start()
        self.actor.velocity.y = self.vy
        self.actor.actor_obj.pos += (0,0.001,0)
        return d

    def update(self, dt):
        if not self.actor.tile:
            return
        maxY = self.actor.tile.boundingBox.maxY
        p = self.actor.actor_obj.pos
        if maxY >= p.y:
            self.actor.actor_obj.pos = (p.x, maxY, p.z)
            self.actor.velocity.y = 0
            self.stop()
            self.deferred.callback(self)
        else:
            pass

class SpinAction(Accellerator):

    acceleration = None

    def __init__(self, actor, velocity=(0,700,0), accel=(0,-450.,0)):
        self.ccw = velocity[1] > 0
        self.angle_acceleration = euclid.Vector3(*accel)
        self.actor = a = actor
        a.angle_velocity = euclid.Vector3(*velocity)


    def tryGoal(self, dt):
        #return tuple(self.actor.angle_velocity) == (0,0,0)
        if self.ccw:
            if self.actor.angle_velocity.y <= 10:
                self.actor.angle_velocity.y = 0
                return True
        else:
            if self.actor.angle_velocity.y >= -10:
                self.actor.angle_velocity.y = 0
                return True


class ElbowSlideAction(Accellerator):

    angle_acceleration = None

    breaking = False
    _goingRight = True
    _atRowSlot = False
    _axis = 'x'

    def __init__(self, actor, row, col, jump=False, accel=(15.,0,0)):

        self.acceleration = euclid.Vector3(*accel)

        self.actor = actor
        self.row = row
        self.col = col
        self.jump = jump
        
        o = self.actor.actor_obj

        self.tgtpos = tgtpos = euclid.Vector3(*scene.getPos(row, col))
        self.startpos = euclid.Vector3(*o.pos)
        a = self.acceleration

        x = tgtpos.x
        self.midx = abs(o.pos.x - x) / 2.

        if x < o.pos.x:
            a.x *= -1
            self.atTargetX = self._atTgtLeft



    def tryGoal(self, dt):

        ## XXX 
        ## Ugghghhhhh this soooooo bad!
        ## :(

        actr = self.actor

        a = self.acceleration
        v = getattr(a, self._axis) * dt
        self._lastv = v
        setattr(actr.velocity, self._axis, getattr(actr.velocity, self._axis) + v)
       
        xf = self._axis == 'x' and (getattr(actr.velocity, self._axis)) or 0
        zf = self._axis == 'z' and (getattr(actr.velocity, self._axis)) or 0
        ##print '(xf=%3.3f, zf=%3.3f)' % (xf, zf)
        ##print '(tgtrow=%d, tgtcol=%d)' % (self.row, self.col)

        o = self.actor.actor_obj
        o.pos += (xf * dt, 0, zf * dt)

        oc = getattr(o.pos, self._axis)
        sc = getattr(self.startpos, self._axis)
        midc = getattr(self, 'mid%s' % self._axis)
        if not self.breaking and abs(oc - sc) >= midc:
            if self.jump:
                print 'jumping %s' % self.actor
                jumpAction = JumpAction(self.actor, 4.)
                jumpAction.start().addCallback(_gencb, self.actor,
                        ).addErrback(_geneb)
            #self.acceleration *= -1
            setattr(self.acceleration, self._axis, getattr(self.acceleration, self._axis) * -1)
            self.breaking = True
            if self._axis == 'x':
                self._goingRight = actr.velocity.x > 0
            else:
                self._goingForward  = actr.velocity.z > 0

        if self.breaking:
            if self._axis == 'x':
                if self._goingRight and actr.velocity.x <= 0:
                    actr.velocity.x = 0.5
                    self.acceleration.x = 0.0
                if not self._goingRight and actr.velocity.x >= 0:
                    actr.velocity.x = -0.5
                    self.acceleration.x = 0.0
            else:
                if self._goingForward and actr.velocity.z <= 0:
                    actr.velocity.z = 0.5
                    self.acceleration.z = 0.0
                if not self._goingForward and actr.velocity.z >= 0:
                    actr.velocity.z = -0.5
                    self.acceleration.z = 0.0

        # Termination clause 1
        if self._axis == 'x' and self.atTargetX():
            #print 'At target X', self.tgtpos.x
            #print 'Actual X', o.pos.x
            self._axis = 'z'
            o.pos = (self.tgtpos.x, o.pos.y, o.pos.z)
            a.x = 0
            a.z = 20.
            z = self.tgtpos.z
            self.midz = abs(o.pos.z - z) / 2.
            self.breaking = False
            actr.velocity.x = 0.
            if z < o.pos.z:
                a.z *= -1
                self.atTargetZ = self._atTgtTop
        elif self._axis == 'z' and  self.atTargetZ():
            #print 'At Z target!', self.tgtpos.z
            #print 'Actual Z', o.pos.z
            actr.velocity.z = 0.
            #o.pos.z = self.tgtpos.z
            o.pos = (o.pos.x, o.pos.y, self.tgtpos.z)
            return True
    
    def _atTgtRight(self):
        return self.actor.actor_obj.pos.x >= self.tgtpos.x
    atTargetX = _atTgtRight

    def _atTgtLeft(self):
        return self.actor.actor_obj.pos.x <= self.tgtpos.x
    
    def _atTgtBottom(self):
        return self.actor.actor_obj.pos.z >= self.tgtpos.z
    atTargetZ = _atTgtBottom

    def _atTgtTop(self):
        return self.actor.actor_obj.pos.z <= self.tgtpos.z


class RookSlideAction(actor.BaseAction):

    acceleration = 20.
    velocity = 0.
    breaking = False

    def __init__(self, actor):
        # generalize this for columns as well
        self.actor = a = actor
        # figure out the furthest tile horizontally
        rows, cols = scene.getFloorDimensions()
        if abs(a.row - (rows - 1)) >= a.row:
            tgtx = rows - 1
        else:
            tgtx = 0
        self.tgtpos = euclid.Vector3(*scene.getPos(tgtx, a.col))
        self.tgtrow = tgtx
        self.tgtcol = a.col

        x = self.tgtpos.x
        o = a.actor_obj
        self.midx = abs(o.pos.x - x) / 2.

        if x < o.pos.x:
            self.acceleration *= -1
            self.atTarget = self._atTgtLeft

        self.startpos = euclid.Vector3(*o.pos)

    def update(self, dt):
        v = self.acceleration * dt
        self._lastv = self.velocity
        self.velocity += v
        
        o = self.actor.actor_obj
        o.pos += (self.velocity * dt, 0, 0)

        
        if not self.breaking and abs(o.pos.x - self.startpos.x) >= self.midx:
            self.acceleration *= -1
            self.breaking = True
            self._goingRight = (self.velocity > 0)

        if self.breaking:
            if self._goingRight and (self.velocity <= 0):
                self.velocity = 0.5
                self.acceleration = 0.0
            if not self._goingRight and (self.velocity >= 0):
                self.velocity = -0.5
                self.acceleration = 0.0

        # Termination clause
        if self.atTarget():
            o.pos.x = self.tgtpos.x
            a = self.actor
            self.stop()
            self.deferred.callback(self)


    def _atTgtRight(self):
        return self.actor.actor_obj.pos.x >= self.tgtpos.x
    atTarget = _atTgtRight

    def _atTgtLeft(self):
        return self.actor.actor_obj.pos.x <= self.tgtpos.x



def _gencb(actor, action):
    print '%r completed action %r' % (actor, action)

def _geneb(reason):
    reason.printTraceback()

#def doRookSlide(actor=None):
#    actor = actor or env.window.actor
#
#    action = RookSlideAction(actor)
#    return actor.act(action).addCallback(_gencb, action
#            ).addErrback(_geneb)

def doRookSlideHoriz(actor=None):
    actor = actor or env.window.actor
    rows, cols = scene.getFloorDimensions()
    if abs(actor.row - (rows - 1)) >= actor.row:
        tgtx = rows - 1
    else:
        tgtx = 0
    action = ElbowSlideAction(actor, tgtx, actor.col,
            jump=True, accel=(5,0,0))
    return actor.act(action).addCallback(_gencb, action
            ).addErrback(_geneb)


def doRookSlideVert(actor=None):
    actor = actor or env.window.actor
    rows, cols = scene.getFloorDimensions()
    if abs(actor.col - (cols - 1)) >= actor.col:
        tgty = cols - 1
    else:
        tgty = 0
    action = ElbowSlideAction(actor, actor.row, tgty,
            jump=False, accel=(0,0,15.))
    return actor.act(action).addCallback(_gencb, action
            ).addErrback(_geneb)


def doJump(actor=None, noact=False, velocity=None):
    actor = actor or env.window.actor

    vy = velocity
    if vy is None:
        vy = 3 + random.random() * 5
    action = JumpAction(actor, vy)
    print 'jumping %r, %r' % (actor, action)

    def resetAltitute(result):
        o = actor.actor_obj
        actor.velocity.y = 0
        o.pos = (o.pos.x, actor.tile.boundingBox.maxY, o.pos.z)
        print 'reset altitude'
    if not noact:
        return actor.act(action).addCallback(_gencb, action
            ).addErrback(_geneb).addBoth(resetAltitute)
    return action.start()


def doJumpSpin(actor=None, noact=False, velocity=None):
    actor = actor or env.window.actor

    d = doJump(noact=noact, velocity=velocity)
    action = SpinAction(actor)
    action.start().addCallback(_gencb, actor
            ).addErrback(_geneb)
    return d

def doElbowSlide(actor=None):
    actor = actor or env.window.actor

    rows, cols = scene.getFloorDimensions()
    r = random.randint(0, rows-1)
    c = random.randint(0, cols-1)

    b = random.choice([False, False])

    action = ElbowSlideAction(actor, r, c, jump=b)
    
    return actor.act(action).addCallback(_gencb, action
            ).addErrback(_geneb)



def doSpinAction(actor=None):
    actor = actor = env.window.actor

    action = SpinAction(actor)
    return actor.act(action).addCallback(_gencb, action
            ).addErrback(_geneb)

def doDash(actor=None):
    actor = actor or env.window.actor

    rows, cols = scene.getFloorDimensions()
    r = random.randint(0, rows-1)
    c = random.randint(0, cols-1)
    
    r, c = random.choice([(r, actor.col), (actor.row, c)])
    #c = actor.col

    #b = random.choice([True, False])
    b = False

    action = ElbowSlideAction(actor, r, c, jump=b)
    
    return actor.act(action).addCallback(_gencb, action
            ).addErrback(_geneb)


def doIt(actor=None,row=1,col=1):
    actor = actor or env.window.actor
    print 'actor', actor
    action = ElbowSlideAction(actor, row, col)
    return actor.act(action).addCallback(_gencb, action
            ).addErrback(_geneb)
