from miru.environment import env
from miru import common

from roboto import actor
from roboto import scene

clock = scene.scheduler

import euclid
import random

class SearchAction(actor.BaseAction):

    vel = 0.3
    radius = 1

    def __init__(self, actor):
        self.actor = actor
        self.steps = self.getSearchSteps()
        self.ctgtrow, self.ctgtcol = self.steps.pop(0)
        self.ctgtpos = euclid.Vector3(*scene.getPos(self.ctgtrow, self.ctgtcol))

    def getSearchSteps(self):
        a = self.actor
        rows, cols = scene.getFloorDimensions()
        steps = []
        if a.row > 0 and a.col > 0 and a.row < rows-1 and a.col < cols-1:
            steps = [ (a.row + 1, a.col + 1), 
                      (a.row + 1, a.col    ), 
                      (a.row + 1, a.col - 1),
                      (a.row    , a.col + 1), 
                      (a.row    , a.col - 1), 
                      (a.row - 1, a.col - 1), 
                      (a.row - 1, a.col    ),
                      (a.row - 1, a.col + 1),
                    ]
        elif a.row == rows-1 and a.col == cols-1:
            steps = [ (a.row    , a.col - 1),
                      (a.row - 1, a.col - 1),
                      (a.row - 1, a.col    ),  
                    ]
        elif a.row == 0 and a.col == 0:
            steps = [ (a.row    , a.col + 1), 
                      (a.row + 1, a.col + 1), 
                      (a.row + 1, a.col    ),  
                    ]
        elif a.row == 0 and a.col == cols - 1:
            steps = [ (a.row + 1, a.col    ), 
                      (a.row + 1, a.col - 1), 
                      (a.row    , a.col - 1), 
                    ] 
        elif a.col == 0 and a.row == rows - 1:
                    [ (a.row    , a.col + 1), 
                      (a.row - 1, a.col + 1), 
                      (a.row - 1, a.col    ),
                    ]
        elif a.row == 0 and a.col > 0 and a.col < cols-1:
            steps = [ (a.row    , a.col + 1), 
                      (a.row + 1, a.col + 1), 
                      (a.row + 1, a.col    ), 
                      (a.row + 1, a.col - 1), 
                      (a.row    , a.col - 1),  
                    ]
        elif a.col == 0 and a.row > 0 and a.row < rows-1:
            steps = [ (a.row + 1, a.col    ), 
                      (a.row + 1, a.col + 1), 
                      (a.row    , a.col + 1), 
                      (a.row - 1, a.col + 1), 
                      (a.row - 1, a.col    ),
                    ]
        elif a.row == rows-1 and a.col > 0 and a.col < cols - 1:
            steps = [ (a.row    , a.col + 1), 
                      (a.row - 1, a.col + 1), 
                      (a.row - 1, a.col    ), 
                      (a.row - 1, a.col - 1), 
                      (a.row    , a.col - 1), 
                    ]  
        elif a.col == cols-1 and a.row > 0 and a.row < rows - 1:
            steps = [ (a.row + 1, a.col    ), 
                      (a.row + 1, a.col - 1), 
                      (a.row    , a.col - 1), 
                      (a.row - 1, a.col - 1), 
                      (a.row - 1, a.col    ),
                    ]
        return steps
    
    def update(self, dt):
        o = self.actor.actor_obj
        v = self.actor.velocity
        vel = self.vel
 
        if o.pos.x != self.ctgtpos.x:
            if o.pos.x < self.ctgtpos.x:
                v.x = self.vel
            elif o.pos.x > self.ctgtpos.x:
                v.x = -self.vel
        elif o.pos.z != self.ctgtpos.z:
            if o.pos.z < self.ctgtpos.z:
                v.z = self.vel
            elif o.pos.z > self.ctgtpos.z:
                v.z = -self.vel

        o.pos += (v.x, v.y, v.z)

        if self.atTargetX():
            o.pos = (self.ctgtpos.x, o.pos.y, o.pos.z)
            v.x = 0.0

        if self.atTargetZ():
            o.pos = (o.pos.x, o.pos.y, self.ctgtpos.z)
            v.z = 0.0

        if self.atTargetX() and self.atTargetZ():
            if len(self.steps) > 0:
                self.ctgtrow, self.ctgtcol = self.steps.pop(0)
                self.ctgtpos = euclid.Vector3(*scene.getPos(self.ctgtrow, self.ctgtcol))
            else:
                clock.unschedule(self.update)
                self.stop()
                if not self.deferred.called:
                    self.deferred.callback(self)

    def atTargetX(self):
        o = self.actor.actor_obj
        if o.pos.x > self.ctgtpos.x - self.vel and o.pos.x < self.ctgtpos.x + self.vel:
            return True
    def atTargetZ(self):
        o = self.actor.actor_obj
        if o.pos.z > self.ctgtpos.z - self.vel and o.pos.z < self.ctgtpos.z + self.vel:
            return True


class RandomWalkAction(actor.BaseAction):

    vel = 0.3
  
    def __init__(self, actor, maxsteps=3):
        self.numsteps = random.randint(1, maxsteps)
        self.actor = actor
        rows, cols = scene.getFloorDimensions()
        self.steps = [(random.randint(0, rows-1), random.randint(0,cols-1)) \
                          for s in range(self.numsteps)] 
        self.ctgtrow, self.ctgtcol = self.steps.pop(0)
        self.ctgtpos = euclid.Vector3(*scene.getPos(self.ctgtrow, self.ctgtcol))

    def update(self, dt):
        o = self.actor.actor_obj
        v = self.actor.velocity
        vel = self.vel
        if o.pos.x != self.ctgtpos.x:
            if o.pos.x < self.ctgtpos.x:
                v.x = self.vel
            elif o.pos.x > self.ctgtpos.x:
                v.x = -self.vel
        elif o.pos.z != self.ctgtpos.z:
            if o.pos.z < self.ctgtpos.z:
                v.z = self.vel
            elif o.pos.z > self.ctgtpos.z:
                v.z = -self.vel

        o.pos += (v.x, v.y, v.z)        

        if self.atTargetX():
            o.pos = (self.ctgtpos.x, o.pos.y, o.pos.z)
            v.x = 0.0

        if self.atTargetZ():
            o.pos = (o.pos.x, o.pos.y, self.ctgtpos.z)
            v.z = 0.0

        if self.atTargetX() and self.atTargetZ():
            if len(self.steps) > 0:
                self.ctgtrow, self.ctgtcol = self.steps.pop(0)
                self.ctgtpos = euclid.Vector3(*scene.getPos(self.ctgtrow, self.ctgtcol))
            else:
                clock.unschedule(self.update)
                self.stop()
                if not self.deferred.called:
                    self.deferred.callback(self) 

    def atTargetX(self):
        o = self.actor.actor_obj
        if o.pos.x > self.ctgtpos.x - self.vel and o.pos.x < self.ctgtpos.x + self.vel:
            return True

    def atTargetZ(self):
        o = self.actor.actor_obj
        if o.pos.z > self.ctgtpos.z - self.vel and o.pos.z < self.ctgtpos.z + self.vel:
            return True

def _gencb(actor, action):
    print '%r completed action %r' % (actor, action)

def _geneb(reason):
    reason.printTraceback()

def doRandWalk(actor=None):
    actor = actor or env.window.actor
    action = RandomWalkAction(actor)
    return actor.act(action).addCallback(_gencb, action).addErrback(_geneb)

def doSearch(actor=None):
    actor = actor or env.window.actor
    action = SearchAction(actor)
    return actor.act(action).addCallback(_gencb, action).addErrback(_geneb)
