from miru import track
from miru import mesh
from miru import imiru
from miru import camera
from miru.environment import env

from roboto import scene
from roboto import tile
from roboto import actor
clock = scene.scheduler

from roboto.actions import df

from roboto import events

import os
P = os.path.join


# Some custom tile loaders for the game defined below

class BoingTile(tile.Tile):

    boinging = False

    def on_enter(self, actor, action=None):
        tile = actor.tile
        if actor.actor_obj.pos.y < tile.boundingBox.maxY:
            self.on_entering(actor, action)

    def on_entering(self, actor, action=None):

        if BoingTile.boinging:
            return
        BoingTile.boinging = True
        print 'BOING!!!!'

        if actor.current_action:
            actor.current_action.stop()
            if not actor.current_action.deferred.called:
                actor.current_action.deferred.callback(actor.current_action)

        def cb(action):
            BoingTile.boinging = False
            print 'Boing action completed', action

        def eb(reason):
            BoingTile.boinging = False
            reason.printTraceback()
      
        rows, cols = scene.getFloorDimensions()

        print 'TILE row=%d, col=%d' % (self.row, self.col)
        v = actor.velocity
        row = (v.x > 0 and self.row - 1) or (v.x and self.row + 1 or actor.row)
        if row < 0:
            row = 0
        elif row >= rows:
            row = rows - 1
        if row == self.row:
            col = (v.z > 0 and self.col + 1) or (v.z < 0 and self.col - 1 or actor.col)
            if col < 0:
                col = 0
            elif col >= cols:
                col = cols - 1
        else:
            col = self.col
        print 'computed row=%d, col=%d' % (row, col)
        actor.velocity.x = 0
        actor.velocity.z = 0
        bounce = df.ElbowSlideAction(actor, row, col)
        bounce.start().addCallbacks(cb, eb)
        actor.current_action = None



def loadBoing(key, row, col):
    tileobj = mesh.loadObj(P('data', 'tile-brushed.obj'))
    mushyobj = mesh.loadObj(P('data', 'mushy.obj'))
    obj = mesh.Group(mushyobj, tileobj)
    obj.tile = BoingTile(obj, row, col)
    return obj

scene.registerLoader('boing', loadBoing)

class GoalTile(tile.Tile):
    
    def on_enter(self, actor, action=None):
        print "Got goal entered..."
        #if not env.window.paused:
        #    print "window is not paused"
        if action: 
             action.stop()
             clock.unschedule(action.update)
             print "last action stopped...."
        clock.schedule(self.finished_goal_animation)

    #def on_entering(self, actor, action=None):
    #    print "Got goal entering.... "
    #    if not env.window.paused:
    #        clock.schedule(self.finsished_goal_animation)
    
    def finished_goal_animation(self, _):
        events.levelTransitionEvent.transition()


class Pulsater(object):

    v = -2

    def __init__(self, light):
        from roboto import scene
        self.light = light
        scene.scheduler.schedule(self.update)

    def update(self, dt):
        a = self.light.ldifs[0]
        a += self.v * dt
        if a < 0:
            a = 0
            self.v *= -1
        elif a > 1:
            a = 1
            self.v *= -1
        self.light.ldifs[:] = (a, a, self.light.ldifs[2], 200)

def loadGoal(key, row, col):
    tileobj = obj = mesh.loadObj(P('data', 'tile-goal.obj'))
    obj = tileobj


    def addLight():
        pl = camera.PositionalLight(ambient=(1,1,0,1), diffuse=(1,1,0,20),
                pos=(obj.pos.x, 0.1, obj.pos.z), debug=True)
        env.camera.lights.append(pl)
        pl.pulsater = Pulsater(pl)

    scene.registerLevelImportCallback(addLight)
    obj.tile = GoalTile(obj, row, col)
    return obj

scene.registerLoader('goal', loadGoal)

class _LinearMoveTo(actor.BaseAction):
    
    def __init__(self, actor, targets=(None,1,None), velocity=(0,20,0)):
        self.obj = actor.actor_obj
        self.actor = actor
        self.targets = targets
        self.velocity = velocity

    def update(self, dt):
        v = self.obj.velocity
        self.obj.velocity = (v.x, self.velocity.y, v.z)
        self.stop()
        self.deferred.callback(self)


class SproingTile(tile.Tile):

    sproinging = False

    def on_hover(self, actor, action=None):
        from roboto.actions import df
        tile = actor.tile
        if not self.sproinging:
            self.sproinging = True
            df.doJumpSpin(noact=True, velocity=3.3).addBoth(self._unsproingify)

    def _unsproingify(self, result):
        self.sproinging = False


def loadSproing(key, row, col):
    print 'Loading sproing'
    tileobj = obj = mesh.loadObj(P('data', 'mushy.obj'))
    obj.tile = SproingTile(obj, row, col)
    obj.pos -= (0, obj.boundingBox.height - 0.15, 0)
    return obj

scene.registerLoader('sproing', loadSproing)

