from pyglet import event
from pyglet import clock

from miru.environment import env

from roboto import scene

from twisted.internet import defer

class TileTransitionAnimation:

    acelleration = 100
    velocity = 0
    tgt_delta = 105

    def __init__(self, tile, delay=0, direction=1,
            tgt_delta=None):
        self.tgt_delta = (tgt_delta, TileDropOutAnimation.tgt_delta)[tgt_delta is None]
        self.tile = tile
        self.direction = direction
        self._start = self.tile.pos.y
        self.deferred = defer.Deferred()
        
        if delay:
            def start():
                clock.schedule(self.update)
            clock.schedule_once(start, delay * 0.05)
        else:
            clock.schedule(self.update)

    def update(self, dt):
        v = self.acelleration * dt
        self.velocity += v
        self.tile.pos.y += self.direction * (self.velocity * dt)
        self.tile.pos.x += self.direction * (self.velocity * dt)
        if self.pastTarget():
            self.deferred.callback(1)
            clock.unschedule(self.update)
        
    def pastTarget(self):
        return abs(self.tile.pos.y - self._start) >= self.tgt_delta


class TileDropOutAnimation(TileTransitionAnimation):
    acceleration = 100
    velocity = 0

class TileDropInAnimation(TileTransitionAnimation):
    acceleration = -100
    velocity = 145

class LevelTransition(event.EventDispatcher):

    def transition(self):
        print 'Dispatching on_transition'
        current = env.levels.current

        def switchToNext(result=None, current=None):
            if current:
                print 'destroying', current
                scene.destroyLevel(current, env)
            next = env.levels.next()
            scene.importLevel(next, env)
            env.window.paused = False
            self.dispatch_event('on_transition', current, next)

        if current:
            env.window.paused = True
            self._startAnimation(TileDropOutAnimation, 1, current, switchToNext)
        else:
            switchToNext()


    def _startAnimation(self, animation, polariy, current, callback):
        ds = []
        for group in scene.currentTileGroups():
            direction = -polariy
            if group.drawable.loc == 'right':
                direction = polariy
            unit_delay = 0
            for row in group.drawable.rows:
                for t in row:
                    anim = animation(t, unit_delay, direction=direction)
                    ds.append(anim.deferred)
                    unit_delay += 1
        defer.gatherResults(ds).addBoth(callback, current)


LevelTransition.register_event_type('on_transition')
levelTransitionEvent = LevelTransition()


