from pyglet import options
options['debug_gl'] = False

from roboto import clock as roboto_clock

from pyglet import font
from pyglet import clock
from pyglet import image

from miru import mesh
from miru import imiru
from miru.environment import env
from miru import camera
from miru import track

from roboto import events
from roboto import constants
from roboto import ui
from roboto import hud
from roboto import actor
from roboto import lights as _lights
from roboto import bling as _bling
from roboto import menu

from twisted.internet import reactor
from twisted.python import components

from zope.interface import implementer

import os
P = os.path.join

DEBUG = constants.DEBUG
USE_GLSL = False

w = ui.Window(800, 400, caption="Stochasm - PyWeek6 - Enter The Foo")
w.set_icon(image.load(P('data', 'icon.png')))
w.set_vsync(False)
#env.camera.pos += (3.4,1.4,0.5)
env.camera.pos += (3.4,1.7,0.5)
env.camera.angle = (40.5,0,0)

env.window = w

if USE_GLSL:
    from miru import shader
    from miru import imiru
    from miru import components
    vs = shader.VertexShader(open(P('data', 'tex.vs')).read())
    fs = shader.FragmentShader(open(P('data', 'tex.fs')).read())
    program = shader.createProgram(vs, fs)
    shaderView = shader.ShaderView(program, camera=env.camera)
    components.registerUtility(imiru.IShaderRenderStage, shaderView)
    env.setRenderStages(imiru.IShaderRenderStage,
                    imiru.IDebuggingRenderStage,
                    imiru.IOSDRenderStage)
    env.camera.objects = shaderView.objects

if DEBUG:
    # Setup the on-screen clock display
    clock_display = clock.ClockDisplay(font=font.load('',18), color=(0.,1.,0.,0.5))
    env.osd.addobj(clock_display)


@implementer(imiru.ITranslationHandler)
def _worldobj_on_translate(handle, x, y, dx, dy):
    return
env.handle.accessible = True
#env.handle = None
def _worldobj_on_translate_adpt(_context):
    return _worldobj_on_translate
def _object_on_translate_adpt(obj):
    return imiru.ITranslationHandler(obj.drawable)
components.registerAdapter(_worldobj_on_translate_adpt,
        imiru.IWorldObject,
        imiru.ITranslationHandler)
components.registerAdapter(_object_on_translate_adpt,
        imiru.IObject,
        imiru.ITranslationHandler)

# the 'robot'
robobj = mesh.loadObj(P('data','lilbot.obj'))
env.addobj(robobj)
sstrack = track.SSCameraTrack(env.camera, robobj,\
        env.window, edge=0.07, axes=(1,0,0))


def pushCameraBack(dt):
    y = env.camera.pos.y
    if y > 0.9:
        env.camera.pos = (env.camera.pos.x, y, y - 0.9 + 4.5)
    elif env.camera.pos.z != 4.5:
        env.camera.pos = (env.camera.pos.x, env.camera.pos.y, 4.5)
    if y < 0.9:
        env.camera.pos = (env.camera.pos.x, 0.9, env.camera.pos.z)

postrack = track.PosTrack(env.camera, robobj, axes=(0,1,0))

robotActor = actor.Actor(robobj)
w.actor = robotActor
w.actors = [robotActor]

# setup the levels
events.levelTransitionEvent.transition()


from roboto import tutor
tutor.tutor.show()

def pump():
    dt = clock.tick(True) # polling tick
    w.clear()
    w.dispatch_events()
    env.render()
    w.flip()
    if w.has_exit and reactor.running:
        reactor.stop()

def crash(reason):
    reason.printTraceback()
    reactor.stop()

def main():
    from twisted.internet import task
    gameLoop = task.LoopingCall(pump)
    gameLoop.start(0., True).addErrback(crash)
    clock.schedule(pushCameraBack)
    reactor.run()


def old_main():
    while not w.has_exit:
        pump()

