from miru.environment import env
from miru import imiru
from miru import mesh

from zope.interface import Interface, Attribute
from zope.interface import implements

from twisted.python import failure

TILE_WIDTH = 0.918224
TILE_LENGTH = 0.918224
TILE_SPACING = 0.0000


class ITile(Interface):

    parent = Attribute("""The parent object wrapping this tile.
        type: C{miru.mesh.Object}
        """)
    
    group = Attribute("""The tile group this tile belongs in
        """)

    row = Attribute("""The row coordinate of the tile""")

    col = Attribute("""The col coordinate of the tile""")


    def on_entering(actor):
        """Called when an actor is overlapping a tile - about to enter.

        @param actor: the actor
        """

    def on_enter(actor, action=None):
        """On enter is called when an actor enters the tile.

        @param actor: the actor

        @param action: the current action being executed by the actor
        """

    def on_hover(actor, action=None):
        """On hover is called with each clock tick while the actor
        is on the tile.

        @param actor: the actor

        @param action: the current action being executed by the actor or
            None if no action is being executed
        """

    def on_exit(actor, action=None):
        """On exit is called with each clock tick while the actor
        is on the tile.

        @param actor: the actor

        @param action: the curren action being executed
        """



class Tile(object):
    """
    @param parent: The parent Object
    """

    group = None

    def __init__(self, parent, row, col):
        self.parent = parent
        self.row = row
        self.col = col
    

    def on_entering(self, actor):
        #print '%r is entering %r' % (actor, self)
        pass

    def on_enter(self, actor, action=None):
        #print '%r entered %r (action=%r)' % (actor, self, action)
        pass
    
    def on_exit(self, actor, action=None):
        #print '%r exited %r (action=%r)' % (actor, self, action)
        pass
    
    def on_hover(self, actor, action=None):
        pass
        #print '%r hovered %r (action=%r)' % (actor, self, action)



class TileGroup(object):
    """Drawable for tiles.

    @param grid: The grid of tiles
    """
    
    implements(imiru.IDrawable, imiru.IWorldObject)
    renderStages = (imiru.IWorldRenderStage, imiru.IShaderRenderStage)
    
    def __init__(self, grid, loc='floor', zpol=1, ypol=0):
        self.rows = grid # deprecate this stupid shit
        self.cols = grid # this too
        ################
        self.grid = grid
        self.loc = loc
        x = 0
        for col in self.grid:
            z = 0
            for tile in col:
                tile.pos += (x, ypol*z, zpol*-z)
                #if loc == 'floor':
                #    print 'Tile pos : ', tile.pos
                #    print 'Tile row=%d,col=%d' % (tile.tile.row, tile.tile.col)
                #    print '-' * 50
                z += TILE_WIDTH + TILE_SPACING
            x += TILE_LENGTH + TILE_SPACING

        # flatten ...
        self._objects = []
        [ self._objects.extend(list(col)) for col in self.rows ]

    def draw(self):
        for o in self._objects:
            o.draw()


class TileEventDispatcher(object):

    def start(self):
        from roboto import scene
        scene.scheduler.schedule(self.update)
        print '[start] scheduled?', self
        self.scene = scene
        self._memo = {}

    def update(self, dt):
        from roboto import constants

        rows, cols = self.scene.getFloorDimensions()


        for actor in env.window.actors:

            # NOTE: Do not set row,col.tile in Action.update anymore
            # This should take care of it for you.
           
            o = actor.actor_obj

            if actor.tile and True:
                d = True

                maxy = actor.tile.boundingBox.maxY
                if o.pos.y <= maxy:
                    d = True
                    #print '=' * 50
                    #print 'Object pos', o.pos.y
                #print 'Actor velocity', actor.velocity

                # Apply gravity to the actor
                if o.pos.y > maxy:
                    v = constants.GRAVITY * dt
                    actor.velocity.y += v
                    newy = o.pos.y + (actor.velocity.y * dt)
                    #print 'newy', newy
                    if newy >= maxy:
                        o.pos = (o.pos.x, newy, o.pos.z)
                    elif newy < maxy:
                        o.pos = (o.pos.x, newy, o.pos.z)
                        #print 'Set new position', newy
                    #if o.pos.y < actor.tile.boundingBox.maxY:
                    #    o.pos = (o.pos.x, actor.tile.boundingBox.maxY, o.pos.z)
                if o.pos.y < 0:
                    o.pos = (o.pos.x, 0, o.pos.z)
                    actor.velocity.y = 0
                #if d:
                #    #print 'maxy', maxy
                #if o.pos.y < maxy:
                #    #print 'setting new Y', maxy
                #    o.pos = (o.pos.x, maxy, o.pos.z)
                #    #print 'bumped up [1]'
                #    actor.velocity.y = 0
                #if o.pos.y <= maxy:
                #    o.pos = (o.pos.x, maxy, o.pos.z)
                #    #print 'bumped up'
                #    actor.velocity.y = 0
                #if d:
                #    print 'Object after', o.pos.y
                #    print '-' * 50
            elif not actor.tile:
                try:
                    tile = self.scene.getTile(actor.row, actor.col)
                    actor.tile = tile
                    print 'set tile on actor', tile.boundingBox.maxY
                    o.pos = (o.pos.x, tile.boundingBox.maxY, o.pos.z)
                except:
                    failure.Failure().printTraceback()

            # See if we're entering other tiles
            for offs in ((-1,0), (-1,-1), (0,-1), (1,-1), (1,0), (1,1), (0,1), (-1,1)):
                nrow = actor.row + offs[0]
                ncol = actor.col + offs[1]
                if nrow < 0 or nrow >= rows or ncol < 0 or ncol >= cols:
                    continue
                tile = self.scene.getTile(nrow, ncol)
                #print 'o bbox: %r' % o.boundingBox
                #print 'tile bbox: %r' % tile.boundingBox
                #print '-' * 50
                bb = tile.boundingBox.copy()
                if bb.maxY == bb.minY:
                    bb.maxY += 0.08
                    bb.minY -= 0.08
                if o.boundingBox.overlaps(bb):
                    tile.tile.on_entering(actor)

            crow, ccol = _getRowCol(actor.actor_obj.pos, actor.actor_obj)
            if (crow, ccol) != (actor.row, actor.col):
                try:
                    actor.tile = self.scene.getTile(crow, ccol)
                except:
                    f = failure.Failure()
                    f.printTraceback()
                    continue
                actor.row = crow
                actor.col = ccol
                if actor.tile:
                    actor.tile.tile.on_enter(actor, actor.current_action)
                #print 'updated coord (%d, %d)' % (crow, ccol)



            changed = False
            rowcoltile = self._memo.get(id(actor))
            if not rowcoltile:
                self._memo[id(actor)] = (actor.row, actor.col, actor.tile)
            else:
                row,col,tile = rowcoltile
                if (row,col) != (actor.row, actor.col):
                    self._memo[id(actor)] = (actor.row, actor.col, tile)
                    if tile:
                        tile.tile.on_exit(actor, actor.current_action)
            if actor.tile:
                actor.tile.tile.on_hover(actor, actor.current_action)


tileEventDistpatcher = TileEventDispatcher()


def _getRowCol(pos, actor=None):
    from roboto import scene

    bump = False
    rows, cols = scene.getFloorDimensions()
    x, _y, z = tuple(pos)

    row = int(x / TILE_LENGTH)
    if row >= rows:
        row = rows - 1
        bump = True
    if row < 0:
        row = 0
        bump = True
    col = int(abs(z) / TILE_WIDTH)
    if col >= cols:
        col = cols - 1
        bump = True
    if col < 0:
        col = 0
        bump = True

    if bump and actor:
        actor.pos = scene.getPos(row, col)

    return row, col





