
#
# This is where all the clever word to action conversion happens
#

# slip slap slop            slup slep
# clip clap clop            clup clep
# rip rap                         rop rep rup
# sip sap sop sup         sep
# nip nap                         nop nup nep
# zip zap                         zop zup zep
# flip flap flop            flup flep
# dip                                 dap dop dup dep
# dink dank dunk            donk denk
# think thank thunk     thenk
# rink rank                     ronk runk renk
# snip snap                     snop snup snep
# crick crack crock     cruck creck
# sit sat set                 sot sut

# shine shan shone shun

# - i - is sharp and short
# - a - is heavier and stronger
# - o - is lazy
# - u - is heavy and solid
# - e - is useless (not very moving)

from roboto import actor
from roboto.actions import df,ml,ph
from roboto import scene

onoCache = {}

def resolveOnomatopoeia(word, actor):
    action = onoCache.get(word,None)
    if action:
        return action
    
    action = computeOnomatopoeia(word)
    onoCache[word] = action
    return action

import re
simpleWordPattern = re.compile('([bcdfghijklmnpqrstvwxyz]+)([aeiou]+)([bcdfghijklmnpqrstvwxyz]+)')
preMap = dict(sl=0, cl=1, r=2, s=3, n=4, z=5, fl=6, th=7, sn=8, cr=10)
vowelMap = dict(i=0,a=2,e=4,o=5,u=6)
postMap = dict(k=0,p=1)

def computeOnomatopoeia(word): 
    match = simpleWordPattern.match(word)
    if match:
        pre,vowel,post = match.groups()
        
        if post == 'rl' and 'c' in pre or 'w' in pre: # a curl action
            up = vowel[0] in 'aei'
            hdir = 'c' in pre
            return (ml.Curl, up, hdir)
        #elif post == 'rp':
            
        else:
            r = preMap.get(pre, 0)
            v = vowelMap.get(vowel[0], 0)
            c = v + postMap.get(post, 0)
        
            if scene.isValidLocation(r,c):
                return (df.ElbowSlideAction, r, v+c)
    
    if len(word)%2:
        return (ph.SearchAction,)
    else:
        return (ph.RandomWalkAction,)


